#!/usr/bin/env python2
# install.py
#2010.09.19

"""(un)install helper for schlim, distribution dependent-
If you want to use this, it may well need modifying for your particular
disribution / configuration.
It handles the following flags (1st command-line argument):
    post_install, post_upgrade, pre_remove, pre_upgrade
"""

import os, sys
from py_compile import compile
from shutil import copyfile
from glob import glob
from schlim_conf import *
import locale
encoding = locale.getdefaultlocale()[1]

def install(flag):
    if flag.startswith('post_'):
        # Byte-compile python modules
        compile(os.path.join(schlimdir, 'schlim_gui.py'))

        if flag == 'post_install':
            if os.path.isfile('/etc/inittab.unschlim'):
                out(_("A saved /etc/inittab already exists at /etc/inittab.unschlim,"))
                out(_("No alterations will be made to it."))
            else:
                out(_("Copying /etc/inittab to /etc/inittab.unschlim."))
                copyfile('/etc/inittab', '/etc/inittab.unschlim')

        out(_("Replacing /etc/inittab by schlim version,"))
        copyfile(os.path.join(schlimdir, 'inittab'), '/etc/inittab')
        inittab_message()

        root_profile()

        if HOMESDIR and not os.path.isdir(HOMESDIR):
            out(_("Creating remote user home base directory: %s") % HOMESDIR)
            os.mkdir(HOMESDIR)

        out(_("Any other display/login manager should be disabled!"), '*!* ')

    else:   # (pre_)
        # Remove byte-compiled modules
        for f in glob(os.path.join(schlimdir, '*.pyc')):
            os.remove(f)

        if flag == 'pre_remove':
            if os.path.isfile('/etc/inittab.unschlim'):
                out(_("Restoring /etc/inittab"))
                os.rename('/etc/inittab.unschlim', '/etc/inittab')
                out(_("Removing schlim login manager may necessitate adjustment of /etc/inittab!"), '*!* ')
                inittab_message()

            root_profile(False)


def inittab_message():
    out(_("Please check that /etc/inittab' is correct,"), '*!!* ')
    out(_("otherwise your system may not start properly."), '*!!* ')


def root_profile(installing=True):
    text = ''
    done = False
    skip = False
    pfile = None
    for f in ('.bash_profile', '.profile'):
        f = os.path.join('/root', f)
        if os.path.isfile(f):
            pfile = f
            with open(f) as fh:
                for line in fh:
                    if line.startswith('#*#-schlim-#*#'):
                        skip = False
                        continue
                    if skip:
                        continue
                    if line.startswith('#*#+schlim+#*#'):
                        skip = True
                        done = True
                        if installing:
                            text += profile_code
                    else:
                        text += line
                if installing and not done:
                    text += '\n' + profile_code
                    done = True

    if skip:
        out(_("An incomplete schlim configuration was found in %s") % pfile, '*!!!* ')
        out(_("The file might be corrupted."), '*!!!* ')
        out(_("Saving current version to %s") % (pfile + '.bak'), '*!!!* ')
        copyfile(pfile, pfile + '.bak')

    if installing:
        if done:
            out(_("Configuring root shell profile:"))
        else:
            out(_("No /root/.bash_profile or /root/.profile found"), '*!!!* ')
            out(_("This is probably a configuration error!"), '*!!!* ')
            out(_("Creating a root shell profile:"), '*!!!* ')
            pfile = '/root/.profile'
            text = profile_code

    elif pfile:
        out(_("Removing schlim configuration from root shell profile:"))

    if pfile:
        out('::: ' + pfile)
        with open(pfile, 'w') as fh:
            fh.write(text)


profile_code = """#*#+schlim+#*#
if [ "${XRUN}" == "y" ]; then
    unset XRUN
    echo %s
    exec %s/xlogin.py
fi
#*#-schlim-#*#
""" % (_("Starting schlim login manager"), schlimdir)


def out(text, prefix='*** '):
    if isinstance(text, str):
        text = text.decode(encoding, 'replace')
    print (unicode(prefix) + text).encode(encoding)


if __name__ == '__main__':
    install(sys.argv[1])
